use crate::config::config::STATIC_WS_PWD;
use crate::deport::handle_ws_msg;
use crate::events::{register_client, ClientMessage, Event};
use crate::handles::handshake::handle_handshake;
use crate::handles::heartbeat::heart_resp;
use crate::handles::online_users_update::send_online_users_and_send;
use crate::utils::json_utils::parse_message;
use dashmap::DashMap;
use futures::{SinkExt, StreamExt};
use lazy_static::lazy_static;
use std::time::{Duration, Instant};
use tokio::sync::mpsc::{Receiver, Sender, UnboundedSender};
use tokio::time;
use tokio_tungstenite::accept_hdr_async;
use tungstenite::handshake::server::{Request, Response};
use tungstenite::{Error, Message};

lazy_static! {
    pub static ref ONLINE_USERS: DashMap<String, String> = DashMap::new();
}

pub(crate) async fn handle_client(
    stream: tokio::net::TcpStream,
    event_sender: UnboundedSender<Event>,
    center_to_client_sender: Sender<ClientMessage>,
    mut center_to_client_receiver: Receiver<ClientMessage>,
) -> Result<(), Error> {
    let must_existed_params = ["deviceID", "fromID", "wsPwd"];
    let mut connection_params = None;

    let ws_stream = match accept_hdr_async(stream, |req: &Request, resp| {
        match handle_handshake(req, &must_existed_params, STATIC_WS_PWD) {
            Ok(params) => {
                connection_params = Some(params);
                Ok(resp)
            }
            Err(error_msg) => {
                println!("{}", error_msg);
                let error_resp = Response::builder()
                    .status(400)
                    .header("Content-Type", "text/plain")
                    .body(Some(error_msg))
                    .unwrap();
                Err(error_resp)
            }
        }
    })
    .await
    {
        Ok(ws) => ws,
        Err(e) => {
            println!("WebSocket握手失败: {}", e);
            return Ok(());
        }
    };

    let (mut sender, mut receiver) = ws_stream.split();
    let mut last_heartbeat_time = Instant::now(); // 包装为 Arc<Mutex<Instant>>

    if let Some(params) = connection_params {
        if let Some(from_id) = params.get("fromID") {
            let from_id = from_id.clone();

            // 注册客户端到事件中心
            register_client(from_id.clone(), center_to_client_sender).await;

            // 发送新连接事件
            event_sender
                .send(Event::NewConnection(from_id.clone(), params))
                .unwrap();

            println!("用户 {} 已连接", from_id);

            // 启动事件中心调度处理任务
            let event_task = tokio::spawn({
                let from_id = from_id.clone();
                async move {
                    while let Some(msg) = center_to_client_receiver.recv().await {
                        println!("消息中心：==> 收到消息 {:?}", &msg);
                        match msg {
                            ClientMessage::CmdUpdateOnlineUsers => {
                                println!(
                                    "消息中心：==> 收到 CmdUpdateOnlineUsers 消息 发送给 {}",
                                    &from_id
                                );
                                if let Err(e) =
                                    send_online_users_and_send(&mut sender, &from_id).await
                                {
                                    println!("处理在线用户列表出错了：{:?}", e);
                                    break;
                                }
                            }
                            ClientMessage::SendClientMessage(
                                from_id,
                                client_message,
                            ) => {
                                if let Err(e) = sender.send(Message::text(client_message)).await {
                                    println!("发送给用户id {} 独立消息失败：{:?}", from_id.clone(), e);
                                    break;
                                }
                            }
                        }
                    }
                }
            });

            // 处理 WebSocket 消息和心跳
            loop {
                tokio::select! {
                    // 处理消息接收
                    maybe_msg = receiver.next() => {
                        if let Some(Ok(msg)) = maybe_msg {
                            if msg.is_text() {
                                let message = msg.to_text()?;
                                match parse_message(message) {
                                    Ok(data) => {
                                        match data.msg_type.as_str() {
                                            "Heart" => {
                                                println!("收到客户端心跳消息 {:?}", &data);
                                                last_heartbeat_time = Instant::now();
                                                if let Ok(json_str) = heart_resp(&from_id) {
                                                    event_sender.clone().send(Event::SendClientMessage(from_id.clone(), json_str)).unwrap();
                                                }
                                            },
                                            _ => {
                                                handle_ws_msg(data.msg_type, from_id.clone(), event_sender.clone());
                                            }
                                        }
                                    }
                                    Err(e) => {
                                        println!("解析JSON数据出错: {}", e);
                                    }
                                }
                            }
                        } else {
                            break; // 客户端断开连接
                        }
                    }
                    // 处理心跳超时
                    _ = time::sleep_until(tokio::time::Instant::from(last_heartbeat_time + Duration::from_secs(20))) => {
                        println!("用户id-{} 20秒内没有发送心跳，挂断连接", from_id);
                        // 发送关闭连接事件
                        event_sender.send(Event::CloseConnection(from_id.clone())).unwrap();
                        break;
                    }
                }
            }

            // 发送关闭连接事件
            event_sender
                .send(Event::CloseConnection(from_id.clone()))
                .unwrap();

            // 等待事件中心调度任务结束
            if let Err(e) = event_task.await {
                println!("事件中心调度任务出错: {:?}", e);
            }
        }
    } else {
        println!("无法获取连接参数");
    }

    Ok(())
}
