use crate::handles::redis::{insert_this_connection, remove_this_connection};
use dashmap::DashMap;
use lazy_static::lazy_static;
use std::collections::HashMap;
use tokio::sync::mpsc;

// 定义事件类型
#[derive(Debug)]
pub enum Event {
    NewConnection(String, HashMap<String, String>),
    CloseConnection(String),
    SendClientMessage(String, String),
}

// 定义事件中心发送给客户端的消息类型
#[derive(Debug)]
pub enum ClientMessage {
    CmdUpdateOnlineUsers,
    SendClientMessage(String, String),
}

lazy_static! {
    // 存储每个客户端的发送者，用于事件中心向客户端发送消息
    pub static ref CLIENT_SENDERS: DashMap<String, mpsc::Sender<ClientMessage>> = DashMap::new();
}

// 注册客户端的发送者
pub async fn register_client(
    from_id: String,
    center_to_client_sender: mpsc::Sender<ClientMessage>,
) {
    close_existing_connection(&from_id).await;
    println!("注册用户 {} 前数量 {}", &from_id, CLIENT_SENDERS.len());
    CLIENT_SENDERS.insert(from_id.clone(), center_to_client_sender);
    println!(
        "注册用户 {} 后数量 {}",
        from_id.clone(),
        CLIENT_SENDERS.len()
    );
}

// 关闭之前绑定的 WebSocket 连接
pub async fn close_existing_connection(from_id: &str) {
    CLIENT_SENDERS.remove(from_id);
    if let Err(e) = remove_this_connection(from_id).await {
        println!("从 Redis 中移除用户信息时出错: {}", e);
    }
}

// 处理事件的任务
pub async fn handle_events(mut receiver: mpsc::UnboundedReceiver<Event>) {
    while let Some(event) = receiver.recv().await {
        match event {
            Event::NewConnection(from_id, params) => {
                println!("有新的连接 用户id {} 更新在线用户列表事件触发", from_id);
                // 将该用户的信息插入到 Redis 中
                if let Err(e) = insert_this_connection(&from_id, &params).await {
                    println!("将用户信息插入到 Redis 中时出错: {}", e);
                }
                notify_all_clients_to_update_online_users().await;
            }
            Event::CloseConnection(from_id) => {
                println!("有关闭的连接 用户id {} 更新在线用户列表事件触发", from_id);
                close_existing_connection(&from_id).await;
                notify_all_clients_to_update_online_users().await;
            }
            Event::SendClientMessage(from_id, msg) => {
                let target_sender = CLIENT_SENDERS.get(&from_id).unwrap();
                if let Err(e) = target_sender
                    .send(ClientMessage::SendClientMessage(
                        from_id, msg,
                    ))
                    .await
                {
                    println!("通知对应sender发送ws消息失败 {}", e);
                } else {
                    println!("通知对应sender发送ws消息成功");
                }
            }
        }
    }
}

// 通知所有客户端线程发送 CmdUpdateOnlineUsers 消息
async fn notify_all_clients_to_update_online_users() {
    println!(
        "尝试发送在新用户更新消息体，目前总人数为：{}",
        CLIENT_SENDERS.len()
    );
    for entry in CLIENT_SENDERS.iter() {
        let sender: &mpsc::Sender<ClientMessage> = entry.value();
        let from_id = entry.key();
        if let Err(e) = sender.send(ClientMessage::CmdUpdateOnlineUsers).await {
            println!("通知客户端 {} 更新在线用户列表失败: {:?}", from_id, e);
        } else {
            println!("通知客户端 {} 更新在线用户列表成功 ===> $$$", from_id);
        }
    }
}
